@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from datetime import datetime
 from enum import Enum
 from typing import Type, Generator, get_args, Dict, ClassVar, Any, Tuple, get_type_hints, Union, Callable, List
@@ -10,6 +10,13 @@ from google.cloud import datastore
 from gcp_pilot import exceptions
 
 DEFAULT_PK_FIELD = 'id'
+MAX_ITEMS_PER_OPERATIONS = 500  # Datastore cannot write more than 500 items per call
+
+
+def _chunks(lst, n):
+    """Yield successive n-sized chunks from lst."""
+    for i in range(0, len(lst), n):
+        yield lst[i:i + n]
 
 
 @dataclass
@@ -24,15 +31,24 @@ class MultipleObjectsFound(Exception):
     filters: Dict
 
 
+def _starts_with_operator(lookup_fields, value):
+    field_name = '.'.join(lookup_fields)
+    return [
+        [field_name, '>=', value],
+        [field_name, '<=', f'{value}\ufffd'],
+    ]
+
+
 @dataclass
 class Manager:
-    lookup_operators = {
+    lookup_operators: ClassVar[Dict[str, Union[str, Callable]]] = {
         'eq': '=',
         'gt': '>',
         'gte': '>=',
         'lt': '<',
         'lte': '<=',
         'in': 'in',
+        'startswith': _starts_with_operator,
     }
 
     _client: ClassVar[datastore.Client] = None
@@ -41,11 +57,13 @@ class Manager:
     doc_klass: Type[Document]
     kind: str
 
-    @classmethod
-    def get_client(cls) -> datastore.Client:
-        if not cls._client:
-            cls._client = datastore.Client()
-        return cls._client
+    def get_client(self) -> datastore.Client:
+        if not self._client:
+            self._client = datastore.Client(namespace=self.get_namespace())
+        return self._client
+
+    def get_namespace(self):
+        return self.doc_klass.Meta.namespace
 
     def build_key(self, pk: Any = None) -> datastore.Key:
         if pk:
@@ -70,8 +88,9 @@ class Manager:
 
         # parse lookup args
         for key, value in kwargs.items():
-            field, operator, value = self._build_filter(key=key, value=value)
-            query.add_filter(field, operator, value)
+            all_filters = self._build_filter(key=key, value=value)
+            for field_name, operator, field_value in all_filters:
+                query.add_filter(field_name, operator, field_value)
 
         # prepare iterator
         cursor = None
@@ -133,9 +152,10 @@ class Manager:
             self.get_client().delete(key=self.build_key(pk=pk))
         else:
             keys = [entity.key for entity in self.query()]
-            self.get_client().delete_multi(keys=keys)
+            for chunk in _chunks(keys, MAX_ITEMS_PER_OPERATIONS):
+                self.get_client().delete_multi(keys=chunk)
 
-    def _build_filter(self, key: str, value: Any) -> Tuple[str, str, Any]:
+    def _build_filter(self, key: str, value: Any) -> List[Tuple[str, str, Any]]:
         lookup_fields = []
         operator = None
 
@@ -145,7 +165,9 @@ class Manager:
             if part in self.lookup_operators:
                 if not is_last:
                     raise exceptions.UnsupportedFormatException(f"Unsupported lookup key format {key}")
-                operator = part
+                operator = self.lookup_operators[part]
+                if callable(operator):
+                    return operator(lookup_fields, value)
             elif idx == 0 and part not in self.fields:
                 raise exceptions.ValidationError(
                     f"{part} is not a valid field. Excepted one of {' | '.join(self.fields)}"
@@ -158,7 +180,7 @@ class Manager:
                 f"Querying with OR clause is not supported"
             )
 
-        return '.'.join(lookup_fields), (operator or '='), value
+        return ['.'.join(lookup_fields), (operator or '='), value]
 
     def to_entity(self, obj: Document) -> datastore.Entity:
         entity = datastore.Entity(key=self.build_key(pk=obj.pk))
@@ -179,6 +201,7 @@ class Metadata:
     fields: Dict[str, type]
     doc_klass: Type[EmbeddedDocument]
     pk_field: str = None
+    namespace: str = None
 
     def from_dict(self, data: Dict) -> EmbeddedDocument:
         data = data.copy()
@@ -229,11 +252,11 @@ class Metadata:
             return value
 
         data = {}
-        for field, field_klass in self.fields.items():
+        for field_name, field_klass in self.fields.items():
             if select_fields and field not in select_fields:
                 continue
 
-            raw_value = getattr(obj, field)
+            raw_value = getattr(obj, field_name)
 
             if getattr(field_klass, '_name', '') == 'List':
                 item = [_unbuild(value=i) for i in raw_value]
@@ -245,7 +268,7 @@ class Metadata:
             else:
                 item = _unbuild(value=raw_value)
 
-            data[field] = item
+            data[field_name] = item
 
         return data
 
@@ -271,6 +294,7 @@ class ORM(type):
         new_cls.Meta = Metadata(
             fields=typed_fields,
             doc_klass=new_cls,
+            namespace=getattr(new_cls, '__namespace__', None),
         )
 
         # Manager initialization
@@ -308,7 +332,9 @@ class ORM(type):
     @classmethod
     def _extract_fields(cls, klass: type) -> Dict[str, type]:
         def _ignore(t: str, k: type):
-            return t.startswith('_') or getattr(k, '__origin__', None) == ClassVar
+            is_private = t.startswith('_')
+            is_class_var = getattr(k, '__origin__', None) == ClassVar  # pylint: disable=comparison-with-callable
+            return is_private or is_class_var
 
         hints = get_type_hints(klass)
         typed_fields = {t: k for t, k in hints.items() if not _ignore(t, k)}
